-- this is an edge detection circuit that detects rising edges of an
-- input signal, level; it generates a short pulse that is used to drive
-- and LED on the FPGA board;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

-- design entity declaration of the "edge detection circuit";
-- we'll call it "top_level" to suggest that this is the top most 
-- entity in our hierarchy of this project;
entity top_level is
	port(
    clk_100MHz : in std_logic; -- connected to 100MHz clock pin of FPGA chip;
    reset : in std_logic; -- connected to left push button, BTNL, on board;
    level : in std_logic; -- connected to right push button, BTNR, on board;
    tick : out std_logic -- drives LD0 on board;
  );
end top_level;

-- architecture description;
architecture my_structural of top_level is

  component edge_detect_fsm is
    port (
      clk, reset: in std_logic;
      level: in std_logic;
      tick: out std_logic
    );
  end component;

  component clk_divider is
    port ( 
      CLK_IN : in STD_LOGIC;
      CLK_OUT : out STD_LOGIC
    );
  end component;

  signal clk : std_logic; -- generated clock with period, Tclk, of about 1 second;

  begin

  -- instantiate the two main components making up this circuit;
  clock_divider_inst : clk_divider port map (clk_100MHz, clk);
  edge_detect_fsm_inst : edge_detect_fsm port map (clk, reset, level, tick);

end my_structural;
