-- fourbit_counter.vhd
-- This is a simple 4-bit (Ripple) binary counter made up
-- of four T flip-flops. It also includes a clock divider
-- to bring down the input clock signal from 100MHz to about 1Hz.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- entity declaration;
entity fourbit_counter is
  port ( 
    clk_100MHz : in STD_LOGIC; -- driven by 100 MHz clock signal coming from outside FPGA;
    Q : out STD_LOGIC_VECTOR (3 downto 0)-- drive the first four LEDs on the board;
  );
end fourbit_counter;

-- architecture description;
architecture my_structural of fourbit_counter is

component my_tff
  port ( 
    T : in STD_LOGIC;
    clk : in STD_LOGIC;
    Q, QN : out STD_LOGIC
  );
end component;

component clk_divider
  port ( 
    CLK_IN : in STD_LOGIC;
    CLK_OUT : out STD_LOGIC
  );
end component;

signal all_T, S0, S1, S2, S3 : STD_LOGIC;
signal clk_1Hz : STD_LOGIC;

begin

-- We use signal all_T set to logic '1' to drive
-- input T of all T flip-flops to logic '1'.
all_T <= '1';
 
-- instantiate clock divider component;
clk_divider_instance: clk_divider port map (CLK_IN => clk_100MHz, CLK_OUT => clk_1Hz);

-- instantiate four T flip-flops;
TFF0: my_tff port map (T => all_T, clk => clk_1Hz, Q => Q(0), QN => S0);
TFF1: my_tff port map (T => all_T, clk => S0, Q => Q(1), QN => S1);
TFF2: my_tff port map (T => all_T, clk => S1, Q => Q(2), QN => S2);
TFF3: my_tff port map (T => all_T, clk => S2, Q => Q(3), QN => S3);

end my_structural;
